#ifndef NODE_H
#define NODE_H

struct node;

#include "event.h"
#include "context.h"
#include "history.h"
#include "truststore.h"

#define MAXBLOCKS  1024
#define SCHEDULE_DELAY 1.5
#define MAXTRANSITQUERIES 20
#define GOODTHRESH 0.9

typedef struct blocks {
	int id;
	int nBlocks;
} blocks;

typedef struct cookieQueryInfo{
	int id;
	int saved;
	int prevhop;
	int bestLen;
	int queriesSent;
	struct trustcookie best[MAXADVERTISE];
} cookieQueryInfo;

typedef struct node {
	int id;
	int nMyBlocks;
	int nRequestedBlocks;
	int nFreeSpace;
	int totMyBlocks;
	int totFreeSpace;
	// trust store
	int nLocations;
	int maxLocations;
	struct history ** locations;
	int nTargets;
	int maxTargets;
	struct history ** targets;
	int isGroup;
	int outstandingGets;
	// list of what we are storing for whom
	int nStores;
	double failAt;
	blocks stores[MAXBLOCKS];
	double good;
	int sleeperCount;
	struct truststore *goodcookies;
	struct truststore *badcookies;
	int maxCookies;
	int nTransitQueries;
	cookieQueryInfo *transitQueries;
	int qid;	// query id counter
	int done;
	int jobsDropped;
	int totJobs;
} node;

node * node_create(struct context *,int myblocks, int freespace, int maxstate, double good, int sleepCount);
void node_free(node * );
int node_store_blocks(struct context *, int id, int src_id, int n);
int node_drop_blocks(struct context *, int id, int src_id);
int node_count_stored_blocks(struct context *, int id, int src_id);


void node_join(struct context * ctx, struct event * e, double t);
void node_schedule(struct context * ctx, struct event * e, double t);
void node_handleget(struct context * ctx, struct event * e, double t);
void node_handlegetresp(struct context * ctx, struct event * e, double t);
void node_handlecheck(struct context * ctx, struct event * e, double t);
void node_handlecheckresp(struct context * ctx, struct event * e, double t);
void node_handlesnub(struct context * ctx, struct event * e, double t);
void node_handlejustkidding(struct context * ctx, struct event * e, double t);
void node_print_status(struct context *ctx, int id);

#endif
