#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "pqueue.h"

#define PARENT(x) ((x)/2)
#define LEFT(x) ((x)<<1)
#define RIGHT(x) (((x)<<1)+1)


/**************************************************************************
 * using alg from CLR p 152;
 * 	note we do NOT use index zero; we just start at pq->array[1];
 * 	that way we skip all kinds of off by 1 errors
 */

void heapify(pqueue * pq, int i);
void pqueue_double(pqueue *);

pqueue * pqueue_create(int initSize){
	pqueue *pq;

	pq = malloc(sizeof(pqueue));
	if(pq==NULL){
		perror("pqueue_create:: malloc");
		return NULL;
	}
	pq->array = malloc(sizeof(qelm)*initSize);
	if(pq->array == NULL){
		perror("pqueue_create:: malloc");
		return NULL;
	}
	pq->size=0;
	pq->length=initSize;

	return pq;
}

int pqueue_isempty(pqueue *pq){
	return (pq->size<1);
}

void heapify(pqueue * pq, int i){
	int l,r;
	int largest;
	double tmpK;
	void * tmpD;
	
	assert(pq);
	l = LEFT(i);
	r = RIGHT(i);
	if((l<= pq->size)&&(pq->array[l].key<pq->array[i].key))
		largest=l;
	else largest=i;
	if((r<= pq->size)&&(pq->array[r].key<pq->array[largest].key))
		largest=r;
	if(largest!=i){
		// swap 'em
		tmpD=pq->array[i].data;
		tmpK=pq->array[i].key;
		pq->array[i].data=pq->array[largest].data;
		pq->array[i].key=pq->array[largest].key;
		pq->array[largest].key=tmpK;
		pq->array[largest].data=tmpD;
		heapify(pq,largest);
	}
}

void pqueue_insert(pqueue* pq,void * data,double key){
	int i;
	if((pq->size+1)>=pq->length)
		pqueue_double(pq);
	pq->size++;
	i=pq->size;
	while((i>1)&&(pq->array[PARENT(i)].key>key)){
		pq->array[i].key=pq->array[PARENT(i)].key;	// bump them down
		pq->array[i].data=pq->array[PARENT(i)].data;
		i=PARENT(i);
	}
	pq->array[i].key=key;
	pq->array[i].data=data;
}

void * pqueue_extract(pqueue *pq , double * key){
	void * data;
	assert(pq);
	if(pq->size<1){
		*key=-1;
		return NULL;
	}
	data=pq->array[1].data;
	*key=pq->array[1].key;
	pq->array[1].data=pq->array[pq->size].data;
	pq->array[1].key=pq->array[pq->size].key;
	pq->size--;
	heapify(pq,1);

	return data;

}

/*******************************************************
 * void pqueue_double(pqueue *pq){
 * 	dynamically double the size of the heap
 */

void pqueue_double(pqueue *pq){
	qelm * neoarray;
	/*fprintf(stderr,"pqueue_double: increasing pqueue from %d to %d\n",
			pq->length, pq->length*2);
			*/
	neoarray = malloc(sizeof(qelm)*pq->length*2);
	assert(neoarray);
	memcpy(neoarray,pq->array,pq->length*sizeof(qelm));
	pq->length*=2;
	pq->array=neoarray;
}
/*************************************************************
 * unittests
 */

void pqueue_unit_test(){
	pqueue *pq;
	int i,j;
	double k;
	void *data;
	int t[] = { 6,4,7,3,1,5,0,9,8,2};

	pq = pqueue_create(5);
	for(i=0;i<10;i++)
		pqueue_insert(pq,(void *) &t[i],t[i]);
	j=1;
	printf("pqueue_unit_test:\n--------------------------\n");
	for(i=0;i<10;i++){
		data=pqueue_extract(pq,&k);
		assert(k<j);
		assert(data=&t[(int)k]);
		j++;
		printf("%d,",(int)k);
	}
	printf("\n");
}
