#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>


#include "toposerver.h"
#include "context.h"

toposerver * toposerver_create(int state){
	toposerver * ts;
	ts = malloc(sizeof(toposerver));
	assert(ts);
	ts->nodeIds=malloc(sizeof(int)*state);
	assert(ts->nodeIds);
	ts->maxState=state;
	ts->nIds=0;
	return ts;
}

/*************************************************************/

void toposerver_free(toposerver * ts){
	free(ts->nodeIds);
	free(ts);
}


/*************************************************************
 * 	toposerver_add: take a newly joined node id,
 * 	and add it to the state.  If the state is full,
 * 	replace it with one uniformly at random
 */

void toposerver_add(toposerver * ts, int node){
	int index;
	if(ts->nIds<ts->maxState){
		ts->nodeIds[ts->nIds++]=node;
		return;
	}
	index=random()%ts->nIds;
	ts->nodeIds[index]=node;
}

/************************************************************
 * void toposerver_get(toposerver *, int *list, int len);
 * 	query the topo server for a random set of nodes
 */

void toposerver_get(toposerver * ts, int id, int *list, int * len){
	int i,j,tmp,count;
	// give things a little shuffle
	for(i=0;i<ts->nIds;i++){
		j = random()%ts->nIds;
		tmp = ts->nodeIds[i];
		ts->nodeIds[i]=ts->nodeIds[j];
		ts->nodeIds[j]=tmp;
	}
	*len=MIN(*len,ts->nIds);
	count=0;
	// and then copy the first 'len' of them, but not ourselves
	for(i=0;((i<ts->nIds)&&(count<*len));i++){
		if(ts->nodeIds[i]==id)
			continue;
		list[count++]=ts->nodeIds[i];
	}
	*len=count;
	return;
}
